import easifem.run
import easifem.test
import easifem.setenv
import easifem.install

testParser = easifem.test.parser
runParser = easifem.run.parser
setenvParser = easifem.setenv.parser
installParser = easifem.install.parser
